/* Copyright (c) 2011, S V N Vishwanathan
 * All rights reserved. 
 * 
 * The contents of this file are subject to the Mozilla Public License 
 * Version 1.1 (the "License"); you may not use this file except in 
 * compliance with the License. You may obtain a copy of the License at 
 * http://www.mozilla.org/MPL/ 
 * 
 * Software distributed under the License is distributed on an "AS IS" 
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the 
 * License for the specific language governing rights and limitations 
 * under the License. 
 * 
 * Authors: S V N Vishwanathan (vishy@stat.purdue.edu)
 *          Xinhua Zhang (xinhua.zhang.cs@gmail.com)
 *
*/

#ifndef _APPCTX_HPP_
#define _APPCTX_HPP_

#include "tao.h"
#include "timer.hpp"

// User-defined application context - contains data needed by the
// application-provided call-back routines that evaluate the function,
// and gradient.
typedef struct {

  PetscMPIInt rank;
  PetscMPIInt size;

  /// training dataset and its statistics
  PetscBool libsvm;       // is input data in libsvm format?
  PetscInt dim;            // Dimensions 
  PetscInt m;              // Number of data points 
  PetscInt m_pos;          // Positive points
  PetscInt m_neg;          // Negative point 
  char data_path[PETSC_MAX_PATH_LEN], labels_path[PETSC_MAX_PATH_LEN]; 
  char tdata_path[PETSC_MAX_PATH_LEN], tlabels_path[PETSC_MAX_PATH_LEN]; 

  Vec labels;              
  Mat data;

  /// Test dataset and its statistics
  Vec tlabels;             // Test labels
  Mat tdata;               // Test data
  PetscInt tdim;           // test Dimensions 
  PetscInt tm;             // Number of test data points 

  /// Training parameters
  PetscInt algo;           // Algorithm we are running
  PetscReal lambda;        // Regularization parameter
  PetscReal eps;           // smoothing parameter  
  char algo_string[PETSC_MAX_PATH_LEN]; // "Hinge", "ROCArea", "PRBEP"

  // Variables for intermediate storage 
  Vec fx;                  // Store predictions <w, x>
  Vec fxloc;               // locally stored fx on head node
  Vec grad;                // Store coefficients for the gradient   
  Vec gradloc;             // locally stored gradient on head node 

  Vec labelsloc;           // locally stored labels on head node  
  VecScatter  scatter;     // Scatter context 
  
  // Time the optimization procedure 
  timer opt_timer;
  
  // Time obj and grad calculation 
  timer objgrad_timer;    // timer for computing the obj and gradient

  // Time matrix vector multiplication
  timer matvec_timer;
  
  // For buffered line search only
  PetscBool use_buf_ls;
  PetscBool new_round_ls;
  PetscInt num_fn_eval;
  PetscReal dsq, wsq, dTimesw;
  Vec XMultW;
  Vec XMultDir;
  
} AppCtx;


// Functors for sorting increasingly
template<typename T>
struct indirect_less_than {
  T *array;
  indirect_less_than(T *value) : array(value) {}
  bool operator() (const int &a, const int &b) {return array[a] < array[b];}
};

// Functors for sorting decreasingly
template<typename T>
struct indirect_greater_than {
  T *array;
  indirect_greater_than(T *value) : array(value) {}
  bool operator() (const int &a, const int &b) {return array[a] > array[b];}
};

#endif
