/* Copyright (c) 2011, S V N Vishwanathan
 * All rights reserved. 
 * 
 * The contents of this file are subject to the Mozilla Public License 
 * Version 1.1 (the "License"); you may not use this file except in 
 * compliance with the License. You may obtain a copy of the License at 
 * http://www.mozilla.org/MPL/ 
 * 
 * Software distributed under the License is distributed on an "AS IS" 
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the 
 * License for the specific language governing rights and limitations 
 * under the License. 
 * 
 * Authors: S V N Vishwanathan (vishy@stat.purdue.edu)
 *          Xinhua Zhang (xinhua.zhang.cs@gmail.com)
 *
*/

#include "petscmat.h"
#include <cstdio>
#include <cstring>
#include <cstdlib>
#include <cassert>

#ifndef _LOADFILE_HPP_
#define _LOADFILE_HPP_

typedef struct{
  PetscInt dim;     // Dimensions 
  PetscInt m;       // Number of data points 
  PetscInt maxnnz;  // Maximum number of nonzeros in any row
  PetscInt maxlen;  // Maximum number of characters in any row 
  Vec      *labels;  // Store labels
  Mat      *data;    // Store data

  // These variables are used only for intermediate storage 
  PetscInt      *nnz_array;

  // Add an extra dimension?
  PetscBool flg;

} AppCtx_file;

char* readline(char* line,PetscInt* maxlen,FILE *input);

PetscErrorCode parse_file(FILE* input,AppCtx_file* user);

void parse_line(char* line,PetscInt* idxs,PetscScalar* vals,PetscScalar* label,int m,AppCtx_file* user);

PetscErrorCode assemble_matrix(FILE* input,PetscInt begin,PetscInt end,PetscInt locm,AppCtx_file* user);

PetscErrorCode fill_arrays(FILE* input,AppCtx_file* user);

PetscErrorCode reparse_file(FILE* input,PetscInt* diag_nnz, PetscInt* offdiag_nnz,AppCtx_file* user);

PetscErrorCode fill_arrays_parallel(FILE* input,AppCtx_file* user);


#endif
